const styleTableId = "#styleTable";
const styleModalId = "#styleFormModal";


// region Init Module

function initMessageStylesModule() {
    initializeDataTable(
        styleTableId,
        [
            {
                title: "Name",
                data: "name",
                render: function(data, type, row) {
                    const btn = renderEditColumn(data);
                    return row.auto_created ?
                        $(btn).removeClass("edit-modal").addClass("disabled")[0]
                        : btn;
                }
            },
            {
                title: "Is Embed",
                data: "is_embed",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Embed Colour",
                data: "colour",
                className: "text-center",
                render: renderHexColourColumn
            },
            {
                title: "Is Hyperlinked",
                data: "is_hyperlinked",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Show Author",
                data: "show_author",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Show Timestamp",
                data: "show_timestamp",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Show Images",
                data: "show_images",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Fetch Images",
                data: "fetch_images",
                className: "text-center",
                render: renderBooleanColumn
            },
            {
                title: "Title Mutator",
                data: "title_mutator_detail",
                render: (data, type, row) => renderMutatorColumn(row.title_mutator_detail)
            },
            {
                title: "Description Mutator",
                data: "description_mutator_detail",
                render: (data, type, row) => renderMutatorColumn(row.description_mutator_detail)
            },
            {
                title: "Editable",
                data: "auto_created",
                className: "text-center",
                render: function(data) {
                    const icon = renderBooleanColumn(!data);
                    if (!data) {
                        return icon;
                    }

                    return $(`
                        <span data-bs-trigger="hover focus"
                            data-bs-custom-class="text-center"
                            data-bs-toggle="popover"
                            data-bs-content="This style was created internally, and cannot be altered.">
                            ${icon}
                        </span>
                    `).popover()[0];
                }
            }
        ]
    );
};


// region Load Data

$(document).on("selectedServerChange", async function() {
    await loadMessageStyleData();
});

$(document).on("doDataLoad", async function() {
    await loadMessageStyleData();
});

async function loadMessageStyleData() {
    if (!selectedServer) {
        return;
    }

    setTableFilter(styleTableId, "server", selectedServer.id);
    await loadTableData(styleTableId, "/api/message-styles/", "GET");
}

// region Table Delete Btns

$(styleModalId).find(".modal-del-btn").on("click", async function() {
    $(styleModalId).modal("hide");

    const id = parseInt($(styleModalId).data("primary-key"));
    const style = $(styleTableId).DataTable().row((idx, row) => { return row.id === id }).data();
    const name = sanitise(style.name);

    await confirmationModal(
        `Delete a Message Style`,
        `Do you wish to permanently delete <b>${name}</b>?`,
        "danger",
        "bi-trash3",
        async () => {
            await ajaxRequest(`/api/message-styles/${style.id}/`, "DELETE");
            setTimeout(async () => {
                $(styleTableId).trigger("doDataLoad");
                await loadSubModalOptions(
                    $(subModalId).find('[data-field="message_style"]'),
                    `/api/message-styles/?server=${selectedServer.id}`
                );
            }, 600);
        },
        () => { $(styleModalId).modal("show") }
    );
})

getTableFiltersComponent(styleTableId).find(".table-del-btn").on("click", async function() {
    const rows = getSelectedTableRows(styleTableId);
    const isMany = rows.length > 1;

    for (const row of rows) {
        if (!row.auto_created) {
            continue;
        }

        await okModal(
            "Cannot Delete Style",
            `<b>${row.name}</b> can't be deleted, as it was created by the system, and thus is immutable.`,
            "warning",
            "bi-arrow-return-right",
            null,
        );
        return
    }

    const names = rows.map(row => row.name);
    const namesString = arrayToHtmlList(names, true).prop("outerHTML");
    
    await confirmationModal(
        `Delete ${isMany ? "Many Message Styles" : "a Message Style"}`,
        `Do you wish to permanently delete ${isMany ? "these" : "this"} <b>${names.length}</b> message style${isMany ? "s" : ""}?<br><br>${namesString}`,
        "danger",
        "bi-trash3",
        async () => {
            rows.forEach(async row => {
                await ajaxRequest(`/api/message-styles/${row.id}/`, "DELETE");
            });
    
            setTimeout(async () => {
                $(styleTableId).trigger("doDataLoad");
                await loadSubModalOptions(
                    $(subModalId).find('[data-field="message_style"]'),
                    `/api/message-styles/?server=${selectedServer.id}`
                );
            }, 600);
        },
        null
    )
});


// region New/Edit Modal

$(styleTableId).closest(".js-tableBody").siblings(".js-tableFilters").on("click", ".table-new-btn", async function() {
    await openDataModal(styleModalId, -1);
});

$(styleTableId).on("click", ".edit-modal", async function() {
    let id = $(styleTableId).DataTable().row($(this).closest("tr")).data().id;
    await openDataModal(styleModalId, id, `/api/message-styles/${id}/`);
});

$(styleModalId).on("submit", async function(event) {
    event.preventDefault();
    onModalSubmit(
        $(styleModalId),
        $(styleTableId),
        "/api/message-styles/"
    ).then(async () => {
        await loadSubModalOptions(
            $(subModalId).find('[data-field="message_style"]'),
            `/api/message-styles/?server=${selectedServer.id}`
        );
    });
});


// region Load Mutator Options

$(document).ready(async function() {
    await loadMutatorOptions();
});

async function loadMutatorOptions() {
    let $inputs = $(styleModalId).find('[data-field="title_mutator"], [data-field="description_mutator"]');

    $inputs.val("").change();
    $inputs.prop("disabled", true);

    $inputs.find("option").each(function() {
        if ($(this).val()) {
            $(this).remove();
        }
    });

    const data = await ajaxRequest("/api/message-mutators/?page_size=25", "GET");
    data.results.forEach(mutator => {
        $inputs.append($(
            "<option>",
            {text: mutator.name, value: mutator.id}
        ));
    });

    $inputs.prop("disabled", false);
}
